* Disclaimer:
* This source code is proprietary of 
* Integrated Signal Processing Systems / RWTH-Aachen / Prof. Meyr (ISS)
* It is part of the DSPstone Benchmarking methodology
* The code, or parts of it may be used for non commercial issues
* All commercial uses require the written consent of  ISS

* file:			fir.asm
* author:		Heiner Loellamnn / Andreas Ropers
* date:			27.06.1999
* project:		dsp-stone
* processor:		TMS320C54x
* short description :	the programm evaluates the FIR-response y(k):	
*			y(k) = sum(i=0...N-1)of( x(k-i)*h(i) )
*			while updating the delay line of x

	.global startp
	.global prof
	.global endp
	.mmregs
	
N	.set 16			; number of FIR taps (<=> array-length of h and x) 

	.data
	
x	.word 1			; x(k) .....
	.word 2
	.word 3
	.word 4
	.word 5
	.word 6
	.word 7
	.word 8
	.word 9
	.word 10
	.word 11
	.word 12
	.word 13
	.word 14
	.word 15		
	.word 16		; x(k-(N-1))

h	.word 1			; h(1) ...
	.word 2			
	.word 3
	.word 4
	.word 5
	.word 6
	.word 7
	.word 8
	.word 9
	.word 10
	.word 11
	.word 12
	.word 13
	.word 14
	.word 15
	.word 16		; h(N)

y	.word 0			; y(k)
	
new_x	.word 8			; next input sample x(k)


	.text
	
	STM  #2    , BRC
	STM  #x	   , AR2	; pointer to x(k-1)
	RPTB endp		; only to test the programm (not relevant for benchmark)
startp:	STM  #h+15 , AR3	; pointer to h(N)
	STM  #y	   , AR4	; pointer to y(k)
	STM  #new_x, AR5	; pointer to the new input value x(k)
prof:	STM  #1	   , AR0	; increment value for circular address modification
	STM  #N	   , BK		; block size of the circular buffer

	RPTZ B	   , #N-2	; for (i=1; i<N; i++) { B=0;
	LMS  *AR3- , *AR2+0%	; B = B+h(N-1+i)*x(k-i); }
	MVDD *AR5  , *AR2	; write the new input value x(n) to delay line of x
	LMS  *AR3  , *AR2	; last iteration:  B = B+h(1)x(k)
	STL  B	   , *AR4	; B -> y(k)
endp:	nop			; (no operation is performed)
	
	.end
	
* remark: if this section will be used e.g. as macro, (only) the contens of the auxiliary register
*	  pointing to x (current offset) has to pushed on the STACK before return instruction.  
	
