* Disclaimer:
* This source code is proprietary of 
* Integrated Signal Processing Systems / RWTH-Aachen / Prof. Meyr (ISS)
* It is part of the DSPstone Benchmarking methodology
* The code, or parts of it may be used for non commercial issues
* All commercial uses require the written consent of  ISS

* file:			irr_N_biquads.asm
* author:		Heiner Loellmann / Andreas Ropers
* date:			18.06.1999
* processor:		TMS320C54x
* project:		dsp-stone
* short description:	benchmarking of an one iir biquad
*                       
*
*	The equations of the filter are:
*       wi(n) =    xi(n) - ai1*wi(n-1) - ai2*wi(n-2)
*       yi(n) = bi0*w(n) + bi1*wi(n-1) + bi2*wi(n-2)
*		1 <= i <= N
*
*		             wi(n)
*   x (n)------(-)---------->-|->----bi0--(+)-------> yi(n)
*               A             |            A
*               |           |1/z|          |
*               |             | wi(n-1)    |
*               |             v            |
*               |-<--ai1----<-|->---bi1-->-|
*               |             |            |
*               |           |1/z|          |
*               |             | wi(n-2)    |
*               |             v            |
*               |-<--ai2----<--->---bi2-->-|
* 
*     The values wi(n-1) and wi(n-2) are stored in the array w 
*     (which has the length of 2*N)
* 
* ( remark:	 equation after the elimination of wi 
*		yi(n) = -ai1*yi(n-1)-ai2*yi(n-2)+bi1*xi(n-1)+bi2*xi(n-2) )
*

	.global startp
	.global prof
	.global endp
	.mmregs

N	.set 4

	.data

x	.word 1			; x1(n)
	.word 1	
	.word 1
	.word 1			; x4(n)
	
y	.word 0			; y1(n)
	.word 0
	.word 0
	.word 0			; y4(n)

w	.word 1			; w1(n-1)
	.word 2			; w1(n-2)
	.word 1
	.word 2
	.word 1
	.word 2
	.word 1			; w4(n-1)
	.word 2			; w4(n-2)

a	.word 1			; a1(1)
	.word 2			; a1(2)
	.word 1
	.word 2
	.word 1
	.word 2
	.word 1			; a4(1)
	.word 2			; a4(2)
	
b	.word 1			; b1(0)
	.word 2			; b1(1)
	.word 3			; b1(2)
	.word 1
	.word 2
	.word 3
	.word 1
	.word 2
	.word 3
	.word 1			; b4(0)
	.word 2			; b4(1)
	.word 3			; b4(2)


	.text
startp:	
*	assumption:	SXM=1 FRCT=0 else	
*	SSBX SXM		; SSBX 0,6
	
	STM #x,AR1
	STM #y,AR2
	STM #w,AR3
	STM #a,AR4
	STM #b,AR5
prof:	
	STM #N-1,BRC		; for  i=1..N do loop
	RPTB loop

	         ; wi(n) = xi(n)-ai1*wi(n-1)-ai2*wi(n-2)
	LD  *AR3+  , T		; wi(n-1) -> TREG
	LD  *AR1+  , B		; xi(n) -> B
	MAS *AR4+  , B		; B = B-ai1*TREG
	LD  *AR3-  , T		; wi(n-2) -> TREG
	MAS *AR4+  , B		; B = B-ai2*TREG ( =wi(n) )
	
	         ; yi(n) = bi0*wi(n)+bi1*wi(n-1)+bi2*wi(n-2)
	STL B      , *AR2	; B -> yi (temporary)
	LD *AR2    , T		; yi -> TREG
	MPY *AR5+  , B		; B = bi0*TREG
	LMS *AR3+  , *AR5+	; B = B=bi1*wi(n-1)
	LMS *AR3-  , *AR5+	; B = B+bi2*wi(n-2)
	
		; updating of wi(n-1) and wi(n-2) 
	LD  *AR3   , T		; wi(n-1) -> TREG
	MVDD *AR2  , *AR3+	; wi(n-1)=wi(n)	
	ST T       , *AR3+	; wi(n-2)=wi(n-1)

loop	nop	;STL B,*AR2+	; B -> yi(n)
endp:	
	.end

* remark:	the `reference C-Program` doesn`t store yi to an array !
